#
#-------------------------------------------------------------------------------
# %Z% %M% %I% %G% %Z%
#-------------------------------------------------------------------------------
#
# This perl module contains an empty configuration as default
#
# Default Vendor Interface
#
# (c) Copyright IBM Coporation 2007. All rights reserved.
#
package UserConfig;

# Company Name: IBM Coporation 

@ISA = qw(Exporter);
@EXPORT = qw(postparser_config);

#use diagnostics;
use strict;
use Exporter ();
use Time::Local;
use AudLog;

my %monthHash = qw (01 Jan 02 Feb 03 Mar 04 Apr 05 May 06 Jun 07 Jul 08 Aug 09 Sep 10 Oct 11 Nov 12 Dec);

#-------------------------------------------------------------------------------
my @rules = (

	#--------------------------------------------------------------------
	#Add GMT offset to starttime, generate end time, convert date format
	#--------------------------------------------------------------------
	{
	RULE_TYPE => 'PERLIZE',
	RULE_DESC => 'Calculate Starttime by adding GMT offset',
	INPUT_FILE_DESCRIPTION => [
			'NSMS_BASIC-#-\d+-#-.*-#-I.pif',
			'NSCP_CLOAD-#-\d+-#-.*-#-I.pif',
			'NSYS_FILES-#-\d+-#-.*-#-I.pif',
			'NSQU_QULEN-#-\d+-#-.*-#-I.pif',
			],
	FILENAME_SUFFIX => 'TIMED',
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	HEADER_COUNTERS_OP => sub
		{
			my $h_ref = shift;
			
			#-------------------------------------------
			#Part: Manipulate time by adding GMT offset
			#-------------------------------------------
			my ($hour, $min);
			my $sec = 0;
			my ($gmt_sign, $gmt_hr, $gmt_min);
			my ($day, $month, $year);
			my ($time_in_seconds, $gmt_time_in_seconds);

			if ( $h_ref->{Starttime} =~ /(\d{2}):(\d{2})/ ) {
				$hour = $1;
				$min = $2;
			}

			if ( $h_ref->{GMT_OFFSET} =~ /([+|-])(\d{2}):(\d{2}):\d{2}/ ) {
				$gmt_sign = $1;
				$gmt_hr = $2;
				$gmt_min = $3;
			}

			if ( $h_ref->{Day} =~ /(\d{4})-(\d{2})-(\d{2})/ ) {
				$year = $1;
				$month = $2;
				$day = $3;
			}
			
			#Perl using month from range of 0 to 11
			$month = $month - 1;

			#Convert time into seconds
			LogMess("Time before convert: Second=$sec Minute=$min Hour=$hour Day=$day Month=$month Year=$year", 5);
			$time_in_seconds = timegm($sec, $min, $hour, $day, $month, $year);
			$gmt_time_in_seconds = ( $gmt_hr * 3600 + $gmt_min * 60 );
			if( $gmt_sign eq "-" ) {
				$gmt_time_in_seconds = ($gmt_time_in_seconds * -1);
			}
			LogMess("GMT Time in seconds is $gmt_time_in_seconds", 5);

			#Add GMT offset to localtime
			$time_in_seconds = $time_in_seconds + $gmt_time_in_seconds;

			#Convert back to date format
			($sec, $min, $hour, $day, $month, $year) = gmtime($time_in_seconds);
			
			#Add 1900 to the year
			$year = $year + 1900;

			#Add 1 back to month
			$month = $month + 1;

			#Convert time into 2 digits if it is not
			if ( $hour < 10 ) {
				$hour = "0$hour";
			}
			if ( $min < 10 ) {
				$min = "0$min";
			}
			if ( $month < 10 ) {
				$month = "0$month";
			}
			if ( $day < 10 ) {
				$day = "0$day";
			}
			LogMess("Time convert after GMT: Second=$sec Minute=$min Hour=$hour Day=$day Month=$month Year=$year", 5);

			$h_ref->{Starttime} = "$hour:$min";
			$h_ref->{Day} = "$day$monthHash{$month}$year";

			#------------------------------------
			#Generate Endtime
			#------------------------------------
			my ($start_time_in_minute, $start_hour, $start_min, $end_hour, $end_minute, $endtime);
			my $GP = $h_ref->{Period} / 60; #convert $GP back to minute
			
			$start_time_in_minute = $hour * 60 + $min;
			$start_time_in_minute = $start_time_in_minute + $GP;
			
			$end_hour = $start_time_in_minute / 60;

			#Remove decimal places
			if ( $end_hour =~ /(\d+)\.\d+/ ) {
				$end_hour = $1;
			}

			$end_minute = $start_time_in_minute % 60;
			
			#Change hour if more than 24
			if ( $end_hour >= 24 ) {
				$end_hour = $end_hour - 24;
			}

			#Convert one digit to 2
			if ( $end_hour < 10 ) {
				$end_hour = "0$end_hour";
			}
			if ( $end_minute < 10 ) {
				$end_minute = "0$end_minute";
			}
			
			$endtime = "$end_hour:$end_minute";
			$h_ref->{Endtime} = $endtime;
		
		return 0;	
		}
	},

	#-------------------------------------------------------------
	#Part NSMS_BASIC: Sum and average the value for each counter
	#-------------------------------------------------------------
	{#Accumulate all records into one record
	RULE_TYPE => 'ACCUMULATE',
	RULE_DESC => 'Accumulate counters within 15 minutes',
	INPUT_FILE_DESCRIPTION => ['NSMS_BASIC-#-\d+-#-.*-#-I-#-TIMED.pif'],
	COUNTERS_TO_SORT_ON => [ qw(SMSC_ID) ],
	OUTPUT_BLOCK_NAME => 'NSMS_BASIC',
	NON_ADDITIVE_COUNTERS => [ qw() ],
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	},

	{#Divide the accumulated counter with NUM_IN_SUM
	RULE_TYPE => 'PERLIZE',
	RULE_DESC => 'Average the counters',
	INPUT_FILE_DESCRIPTION => ['NSMS_BASIC-#-\d+-#-.*-#-I-#-TIMED-#-NSMS_BASIC-#-A.pif'],
	FILENAME_SUFFIX => 'PZ',
	OUTPUT_BLOCK_NAME => 'NSMS_BASIC',
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	DATA_COUNTERS_OP => sub 
		{
		my $c_ref = shift;
		my $count = $c_ref->{NSMS_BASIC_NUM_IN_SUM};
		my $acc_value = $c_ref->{cpu_load};
		my $avg_value;
		
		foreach my $c_key (keys %$c_ref) {
			unless (($c_key eq "SMSC_ID") || ($c_key eq "deliveries_mt") || ($c_key eq "deliveries_all")) {
			$c_ref->{$c_key} = $c_ref->{$c_key} / $count;
			}
		}
			
		return 0;
		},
	HEADER_COUNTERS_OP => sub 
		{
		my $h_ref = shift;

		delete $h_ref->{startTime};
		delete $h_ref->{interval};
		delete $h_ref->{GMT_OFFSET};
		delete $h_ref->{measurementType};
		delete $h_ref->{uniqueID};

		return 0;
		}
	},

	#-------------------------------------------------------
	#Part NSCP_CLOAD: Sum and average the value for each counter
	#-------------------------------------------------------
	{#Accumulate all records into one record
	RULE_TYPE => 'ACCUMULATE',
	RULE_DESC => 'Accumulate counters within 15 minutes',
	INPUT_FILE_DESCRIPTION => ['NSCP_CLOAD-#-\d+-#-.*-#-I-#-TIMED.pif'],
	COUNTERS_TO_SORT_ON => [ qw(SMSC_ID NSCP_ID) ],
	OUTPUT_BLOCK_NAME => 'NSCP_CLOAD',
	NON_ADDITIVE_COUNTERS => [ qw() ],
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	},

	{#Divide the accumulated counter with NUM_IN_SUM
	RULE_TYPE => 'PERLIZE',
	RULE_DESC => 'Average the counters',
	INPUT_FILE_DESCRIPTION => ['NSCP_CLOAD-#-\d+-#-.*-#-I-#-TIMED-#-NSCP_CLOAD-#-A.pif'],
	FILENAME_SUFFIX => 'PZ',
	OUTPUT_BLOCK_NAME => 'NSCP_CLOAD',
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	DATA_COUNTERS_OP => sub 
		{
		my $c_ref = shift;
		my $count = $c_ref->{NSCP_CLOAD_NUM_IN_SUM};
		my $acc_value = $c_ref->{cpu_load};
		my $avg_value;

		$avg_value = $acc_value / $count;
		$c_ref->{cpu_load} = $avg_value;

		return 0;
		},
	HEADER_COUNTERS_OP => sub 
		{
		my $h_ref = shift;

		delete $h_ref->{startTime};
		delete $h_ref->{interval};
		delete $h_ref->{GMT_OFFSET};
		delete $h_ref->{measurementType};
		delete $h_ref->{uniqueID};

		return 0;
		}
	},

	#-------------------------------------------------------
	#Part NSYS_FILES: Sum and average the value for each counter
	#-------------------------------------------------------
	{#Accumulate all records into one record
	RULE_TYPE => 'ACCUMULATE',
	RULE_DESC => 'Accumulate counters within 15 minutes',
	INPUT_FILE_DESCRIPTION => ['NSYS_FILES-#-\d+-#-.*-#-I-#-TIMED.pif'],
	COUNTERS_TO_SORT_ON => [ qw(SMSC_ID NSYS_ID) ],
	OUTPUT_BLOCK_NAME => 'NSYS_FILES',
	NON_ADDITIVE_COUNTERS => [ qw() ],
	PRODUCE_LIF => 0,
	PRODUCE_PIF => 'True',
	},
	
	{
	RULE_TYPE => 'PERLIZE',
	RULE_DESC => 'Average the counters',
	INPUT_FILE_DESCRIPTION => ['NSYS_FILES-#-\d+-#-.*-#-I-#-TIMED-#-NSYS_FILES-#-A.pif'],
	FILENAME_SUFFIX => 'PZ',
	OUTPUT_BLOCK_NAME => 'NSYS_FILES',
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	DATA_COUNTERS_OP => sub
		{
		my $c_ref = shift;
		my $count = $c_ref->{NSYS_FILES_NUM_IN_SUM};
		my $c_1 = $c_ref->{free_total_nodes},
		my $c_2 = $c_ref->{free_total_blocks},
		my $avg_value;
		
		$avg_value = $c_1 / $count;
		$c_ref->{free_total_nodes} = $avg_value;

		$avg_value = $c_2 / $count;
		$c_ref->{free_total_blocks} = $avg_value;

		return 0;
		},
	HEADER_COUNTERS_OP => sub 
		{
		my $h_ref = shift;

		delete $h_ref->{startTime};
		delete $h_ref->{interval};
		delete $h_ref->{GMT_OFFSET};
		delete $h_ref->{measurementType};
		delete $h_ref->{uniqueID};

		return 0;
		}
	},

	#-------------------------------------------------------
	#Part NSQU_QULEN: Sum and average the value for each counter
	#-------------------------------------------------------
	{#Accumulate all records into one record
	RULE_TYPE => 'ACCUMULATE',
	RULE_DESC => 'Accumulate counters within 15 minutes',
	INPUT_FILE_DESCRIPTION => ['NSQU_QULEN-#-\d+-#-.*-#-I-#-TIMED.pif'],
	COUNTERS_TO_SORT_ON => [ qw(SMSC_ID NSQU_ID) ],
	OUTPUT_BLOCK_NAME => 'NSQU_QULEN',
	NON_ADDITIVE_COUNTERS => [ qw() ],
	PRODUCE_LIF => 0,
	PRODUCE_PIF => 'True',
	},
	
	{
	RULE_TYPE => 'PERLIZE',
	RULE_DESC => 'Average the counters',
	INPUT_FILE_DESCRIPTION => ['NSQU_QULEN-#-\d+-#-.*-#-I-#-TIMED-#-NSQU_QULEN-#-A.pif'],
	FILENAME_SUFFIX => 'PZ',
	OUTPUT_BLOCK_NAME => 'NSQU_QULEN',
	PRODUCE_PIF => 'True',
	PRODUCE_LIF => 0,
	DATA_COUNTERS_OP => sub
		{
		my $c_ref = shift;
		my $count = $c_ref->{NSQU_QULEN_NUM_IN_SUM};
		my $c_1 = $c_ref->{queue_length},
		my $avg_value;
		
		$avg_value = $c_1 / $count;
		$c_ref->{queue_length} = $avg_value;

		return 0;
		},
	HEADER_COUNTERS_OP => sub 
		{
		my $h_ref = shift;

		delete $h_ref->{startTime};
		delete $h_ref->{interval};
		delete $h_ref->{GMT_OFFSET};
		delete $h_ref->{measurementType};
		delete $h_ref->{uniqueID};

		return 0;
		}
	},

	#-------------------------------------------
	#Part: Batch all perlized pif into one lif
	#-------------------------------------------
	{#Batch pif into one lif
	RULE_TYPE => 'BATCHFILES',
	RULE_DESC => 'Batch the 15 min interval files into one lif',
	INPUT_FILE_DESCRIPTION => [
		'(NSYS_FILES-#-\d+)-#-.*-#-I-#-TIMED-#-NSYS_FILES-#-A-#-PZ.pif',
		'(NSCP_CLOAD-#-\d+)-#-.*-#-I-#-TIMED-#-NSCP_CLOAD-#-A-#-PZ.pif',
		'(NSQU_QULEN-#-\d+)-#-.*-#-I-#-TIMED-#-NSQU_QULEN-#-A-#-PZ.pif',
		'(NSMS_BASIC-#-\d+)-#-.*-#-I-#-TIMED-#-NSMS_BASIC-#-A-#-PZ.pif',
	],
	OUTPUT_FILENAME_START => 'BATCHED',
	PRODUCE_LIF => 'True',
	PRODUCE_PIF => 0,
	},
	
	#{#Produce individual lifs for GTH
	#RULE_TYPE => 'PIF_2_OUTPUT',
	#RULE_DESC => 'Generate LIF for GTH',
	#INPUT_FILE_DESCRIPTION => [
	#	'NSYS_FILES-#-\d+-#-.*-#-I-#-TIMED-#-NSYS_FILES-#-A-#-PZ.pif',
	#	'NSCP_CLOAD-#-\d+-#-.*-#-I-#-TIMED-#-NSCP_CLOAD-#-A-#-PZ.pif',
	#	'NSQU_QULEN-#-\d+-#-.*-#-I-#-TIMED-#-NSQU_QULEN-#-A-#-PZ.pif',
	#	'NSMS_BASIC-#-\d+-#-.*-#-I-#-TIMED-#-NSMS_BASIC-#-A-#-PZ.pif',
	#],
	#PRODUCE_LIF => 'True',
	#PRODUCE_PIF => 0,
	#},

);

#-------------------------------------------------------------------------------

sub postparser_config {
	return \@rules;
}

1;
